package com.adventurer_engine.client.model.tmt;

package .var.folders.cl.9twcgjfx4vsg5zl73c6wxmvm0000gn.T;

import atc;
import com.adventurer_engine.client.model.tmt.Coord2D;
import com.adventurer_engine.client.model.tmt.PositionTextureVertex;
import com.adventurer_engine.client.model.tmt.PositionTransformVertex;
import com.adventurer_engine.client.model.tmt.Shape3D;
import com.adventurer_engine.client.model.tmt.TexturedPolygon;
import java.util.ArrayList;
import ls;

public class Shape2D {
  public ArrayList<Coord2D> coords;
  
  public Shape2D() {
    this.coords = new ArrayList<Coord2D>();
  }
  
  public Shape2D(Coord2D[] coordArray) {
    this.coords = new ArrayList<Coord2D>();
    for (int idx = 0; idx < coordArray.length; idx++)
      this.coords.add(coordArray[idx]); 
  }
  
  public Shape2D(ArrayList<Coord2D> coordList) {
    this.coords = coordList;
  }
  
  public Coord2D[] getCoordArray() {
    return (Coord2D[])this.coords.toArray();
  }
  
  public Shape3D extrude(float x, float y, float z, float rotX, float rotY, float rotZ, float depth, int u, int v, float textureWidth, float textureHeight, int shapeTextureWidth, int shapeTextureHeight, int sideTextureWidth, int sideTextureHeight, float[] faceLengths) {
    PositionTransformVertex[] verts = new PositionTransformVertex[this.coords.size() * 2];
    PositionTransformVertex[] vertsTop = new PositionTransformVertex[this.coords.size()];
    PositionTransformVertex[] vertsBottom = new PositionTransformVertex[this.coords.size()];
    TexturedPolygon[] poly = new TexturedPolygon[this.coords.size() + 2];
    atc extrudeVector = atc.a(0.0D, 0.0D, depth);
    setVectorRotations(extrudeVector, rotX, rotY, rotZ);
    if (faceLengths != null && faceLengths.length < this.coords.size())
      faceLengths = null; 
    float totalLength = 0.0F;
    for (int idx = 0; idx < this.coords.size(); idx++) {
      Coord2D curCoord = this.coords.get(idx);
      Coord2D nextCoord = this.coords.get((idx + 1) % this.coords.size());
      float texU1 = (curCoord.uCoord + u) / textureWidth;
      float texU2 = (shapeTextureWidth * 2 - curCoord.uCoord + u) / textureWidth;
      float texV = (curCoord.vCoord + v) / textureHeight;
      atc vecCoord = atc.a(curCoord.xCoord, curCoord.yCoord, 0.0D);
      setVectorRotations(vecCoord, rotX, rotY, rotZ);
      verts[idx] = new PositionTransformVertex(x + (float)vecCoord.c, y + (float)vecCoord.d, z + (float)vecCoord.e, texU1, texV);
      verts[idx + this.coords.size()] = new PositionTransformVertex(x + (float)vecCoord.c - (float)extrudeVector.c, y + (float)vecCoord.d - (float)extrudeVector.d, z + (float)vecCoord.e - (float)extrudeVector.e, texU2, texV);
      vertsTop[idx] = new PositionTransformVertex((PositionTextureVertex)verts[idx]);
      vertsBottom[this.coords.size() - idx - 1] = new PositionTransformVertex((PositionTextureVertex)verts[idx + this.coords.size()]);
      if (faceLengths != null) {
        totalLength += faceLengths[idx];
      } else {
        totalLength = (float)(totalLength + Math.sqrt(Math.pow(curCoord.xCoord - nextCoord.xCoord, 2.0D) + Math.pow(curCoord.yCoord - nextCoord.yCoord, 2.0D)));
      } 
    } 
    poly[this.coords.size()] = new TexturedPolygon((PositionTextureVertex[])vertsTop);
    poly[this.coords.size() + 1] = new TexturedPolygon((PositionTextureVertex[])vertsBottom);
    float currentLengthPosition = totalLength;
    for (int i = 0; i < this.coords.size(); i++) {
      Coord2D curCoord = this.coords.get(i);
      Coord2D nextCoord = this.coords.get((i + 1) % this.coords.size());
      float currentLength = (float)Math.sqrt(Math.pow(curCoord.xCoord - nextCoord.xCoord, 2.0D) + Math.pow(curCoord.yCoord - nextCoord.yCoord, 2.0D));
      if (faceLengths != null)
        currentLength = faceLengths[faceLengths.length - i - 1]; 
      float ratioPosition = currentLengthPosition / totalLength;
      float ratioLength = (currentLengthPosition - currentLength) / totalLength;
      float texU1 = (ratioLength * sideTextureWidth + u) / textureWidth;
      float texU2 = (ratioPosition * sideTextureWidth + u) / textureWidth;
      float texV1 = (v + shapeTextureHeight) / textureHeight;
      float texV2 = (v + shapeTextureHeight + sideTextureHeight) / textureHeight;
      PositionTransformVertex[] polySide = new PositionTransformVertex[4];
      polySide[0] = new PositionTransformVertex((PositionTextureVertex)verts[i], texU2, texV1);
      polySide[1] = new PositionTransformVertex((PositionTextureVertex)verts[this.coords.size() + i], texU2, texV2);
      polySide[2] = new PositionTransformVertex((PositionTextureVertex)verts[this.coords.size() + (i + 1) % this.coords.size()], texU1, texV2);
      polySide[3] = new PositionTransformVertex((PositionTextureVertex)verts[(i + 1) % this.coords.size()], texU1, texV1);
      poly[i] = new TexturedPolygon((PositionTextureVertex[])polySide);
      currentLengthPosition -= currentLength;
    } 
    return new Shape3D(verts, poly);
  }
  
  protected void setVectorRotations(atc vector, float xRot, float yRot, float zRot) {
    float x = xRot;
    float y = yRot;
    float z = zRot;
    float xC = ls.b(x);
    float xS = ls.a(x);
    float yC = ls.b(y);
    float yS = ls.a(y);
    float zC = ls.b(z);
    float zS = ls.a(z);
    double xVec = vector.c;
    double yVec = vector.d;
    double zVec = vector.e;
    double xy = xC * yVec - xS * zVec;
    double xz = xC * zVec + xS * yVec;
    double yz = yC * xz - yS * xVec;
    double yx = yC * xVec + yS * xz;
    double zx = zC * yx - zS * xy;
    double zy = zC * xy + zS * yx;
    xVec = zx;
    yVec = zy;
    zVec = yz;
    vector.c = xVec;
    vector.d = yVec;
    vector.e = zVec;
  }
}
